<?php
if ( ! defined( 'MEDIAWIKI' ) )
        die();
/* MediaWiki extension to submit changes to Basset for antispam check
 * Copyright (C) 2009  Ian Weller <ian@ianweller.org>
 * Copyright (C) 2012  Patrick Uiterwijk <puiterwijk@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
$wgHooks['ArticleSaveComplete'][] = 'basset_article_save';


// This function may not exist.
// Example implementation from http://php.net/manual/en/function.apache-request-headers.php
if( !function_exists('apache_request_headers') ) {
function apache_request_headers() {
  $arh = array();
  $rx_http = '/\AHTTP_/';
  foreach($_SERVER as $key => $val) {
    if( preg_match($rx_http, $key) ) {
      $arh_key = preg_replace($rx_http, '', $key);
      $rx_matches = array();
      // do some nasty string manipulations to restore the original letter case
      // this should work in most cases
      $rx_matches = explode('_', $arh_key);
      if( count($rx_matches) > 0 and strlen($arh_key) > 2 ) {
        foreach($rx_matches as $ak_key => $ak_val) $rx_matches[$ak_key] = ucfirst($ak_val);
        $arh_key = implode('-', $rx_matches);
      }
      $arh[$arh_key] = $val;
    }
  }
  return( $arh );
}
}



function basset_article_save(
  &$article,
  &$user,
  $text,
  $summary,
  $minoredit,
  $watchthis,
  $sectionanchor,
  &$flags,
  $revision,
  &$status,
  $baseRevId
) {
    global $basset_url, $basset_username, $basset_password;

    $title = $article->getTitle();
    if ( $title->getNsText() ) {
        $titletext = $title->getNsText() . ":" . $title->getText();
    } else {
        $titletext = $title->getText();
    }

    if ( is_object($revision) ) {
        $revurl = $title->getFullURL('diff=prev&oldid=' . $revision->getId());
    } else {
        $revurl = $title->getFullURL();
    }

    $data = array(
        'action' => 'mediawiki.edit',
        'time' => time(),
        'data' => array(
            'page' => array(
                'url' =>$title->getFullURL(),
                'title' => $titletext,
                'revisionurl' => $revurl,
                'user' => $user->getName(),
                'summary' => $summary,
                'minoredit' => $minoredit,
                'revision' => $revision,
                'baserevision' => $baseRevId,
            ),
            'request_headers' => apache_request_headers()
        )
    );

    $encodedData = json_encode($data);
    $ch = curl_init($basset_url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_HEADER, 1);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $encodedData);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));

    curl_setopt($ch, CURLOPT_USERPWD, $basset_username . ":" . $basset_password);

    $result = curl_exec($ch);
    if($result === FALSE) {
        error_log("Spamcheck submission failed for: " . $title->getFullURL());
        error_log("Request failed");
        return true;
    };
    $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
    $header = substr($result, 0, $header_size);
    $body = substr($result, $header_size);

    curl_close($ch);

    $decoded = json_decode($body);
    if($decoded === null || $decoded->result != 'checking') {
        error_log("Spamcheck submission failed for: " . $title->getFullURL());
        error_log("Was unable to request spamcheck: " . $body);
    }

    return true;
}


$wgExtensionCredits['antispam'][] = array(
        'path' => __FILE__,
        'name' => 'Basset Submitter',
        'author' => 'Patrick Uiterwijk',
        'description' => 'Submits all changes to a Basset server',
        'url' => 'https://pagure.io/basset/',
        'version' => '0.3',
);
?>
